<?php

namespace Drupal\sms_link_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'sms_link_formatter'.
 *
 * @FieldFormatter(
 *   id = "sms_link_formatter",
 *   label = @Translation("Link with SMS support"),
 *   field_types = {
 *     "sms_link"
 *   }
 * )
 */
class SmsLinkFormatter extends LinkFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      if (!empty($item->uri)) {
        // Handle 'sms:' scheme manually
        $uri = $item->uri;
        $title = $item->title ?: $item->uri;

        // Check for sms scheme manually and create renderable link without Url object
        if (str_starts_with($uri, 'sms:')) {
          $elements[$delta] = [
            '#type' => 'link',
            '#title' => $title,
            '#url' => $uri,
            '#options' => ['external' => TRUE],
          ];
          // But since #url must be Url object, fallback to plain render array:
          $elements[$delta] = [
            '#type' => 'html_tag',
            '#tag' => 'a',
            '#attributes' => ['href' => $uri],
            '#value' => $title,
          ];
        }
        else {
          // Normal handling for http, https, mailto
          $url = Url::fromUri($uri, ['external' => TRUE]);
          $elements[$delta] = [
            '#type' => 'link',
            '#title' => $title,
            '#url' => $url,
          ];
        }
      }
    }

    return $elements;
  }
}
